
/* *****************************************************************************
* ******************************************************************************

Simulations for "Next please! Estimating the effect of treatments allocated
by randomized waiting lists"

*/

clear all
set more off
set matsize 2100
global CHEMIN="C:\Users\l.behaghel\Documents\Waiting list"
*global CHEMIN="C:\waiting list"
*global CHEMIN="U:\waiting list"
*global CHEMIN="C:\Users\Economics\Dropbox\Waiting list"
*global CHEMIN="C:\Users\Clement\Dropbox\Waiting list"
global TIME=substr("${S_TIME}",1,2)+"_"+substr("${S_TIME}",4,2)
capture log close
log using "$CHEMIN\log\simulations ${S_DATE} ${TIME}.log", replace

capture prog drop simu
prog define simu, rclass

	* *** Initiate base
	global N=${K}*${Nk}
	clear
	set obs ${N}

	* **** Lottery
	gen id=_n
	gen LOTTERY=int((id-1)/${Nk}) + 1

	* **** Takers

	global takers=$compliers+$alwaystakers
	gen modulus=mod(id-1,${Nk})
	gen Comp=(modulus<=$compliers -1)
	gen AT=(modulus>=$compliers&modulus<$compliers+$alwaystakers)
	gen NT=1-Comp-AT
	gen C=Comp+AT
	sum Comp AT C NT

	* *** Instruments

	* Random waiting lists
	gen u=runiform()
	egen RANK=rank(u), by (LOTTERY)

	* L (rank of last student admitted)
	forvalue a=`=$Sk'/`=$Sk+$Nk-$takers'{
	gen C`a'=C*(RANK<=`a')
	egen SUM_C`a'=sum(C`a'), by(LOTTERY)
	}
	gen L=0 
	forvalue a=`=$Sk'/`=$Sk+$Nk-$takers'{
	replace L=`a' if L==0 & SUM_C`a'==$Sk
	}

	* Initial offer
	gen ZI=(RANK<=$Sk)

	* Ever offer instrument
	gen ZE=(RANK<=L)

	* *** Treatment
	gen D=ZE*C+(1-ZE)*AT

	* *** Outcome variables

	* Potential outcomes
	gen y0=invnorm(runiform())+$delta_NT*NT+$delta_AT*AT
	gen y1=y0+0.2
	
	if $DGP==3 {
	replace y1=y1+invnorm(runiform())
	}
	
	* Observed outcome
	gen Y=y0*(1-D)+y1*D


	* *** Weights

	* w: downweighting takers by one unit among applicants receiving an offer
	gen w=1-ZE*D/$Sk

	* wDR: weights for DREO estimator
	egen sw=total(w)
	egen swk=total(w), by(LOTTERY)

	gen wZ=w*ZE
	egen swZ=total(wZ)
	egen swZk=total(wZ), by(LOTTERY)

	gen w1mZ=w*(1-ZE)
	egen sw1mZ=total(w1mZ)
	egen sw1mZk=total(w1mZ), by(LOTTERY)

	gen wDR=w * ( ZE*(swZ/sw)*(swk/swZk) + (1-ZE)*(sw1mZ/sw)*(swk/sw1mZk) )*${Nk}/(${Nk}-1)
	drop sw*

	* *** 2SLS

	* Initial offer with lottery dummies
	ivregress 2sls Y i.LOTTERY (D=ZI), r 
	return scalar tslsI=_b[D]
	return scalar coverageI=(0.2>=_b[D]-1.96*_se[D]&0.2<=_b[D]+1.96*_se[D])
	* Ever offer with lottery dummies
	ivregress 2sls Y i.LOTTERY (D=ZE), r
	return scalar tslsE=_b[D]
	return scalar coverageE=(0.2>=_b[D]-1.96*_se[D]&0.2<=_b[D]+1.96*_se[D])
	* Doubly reweighted
	ivregress 2sls Y (D=ZE) [pw=wDR], r
	return scalar tslsDRE=_b[D]
	scalar delta=_b[D]
	* se DREO
	reg D ZE [pw=wDR]
	scalar fs=_b[ZE]
	gen lambda_k=.
	forvalue j=1/`=$K'{
	reg D ZE [pw=wDR] if LOTTERY==`j'
	scalar fs_k=_b[ZE]
	reg Y ZE [pw=wDR] if LOTTERY==`j'
	scalar rf_k=_b[ZE]
	replace lambda_k=1/fs*(rf_k-fs_k*delta) in `j'
	}
	sum lambda_k
	scalar se_DREO=r(sd)/sqrt(r(N)-1)
	drop lambda_k
	return scalar coverageDRE=(0.2>=delta-1.96*se_DREO&0.2<=delta+1.96*se_DREO)

end

* **************************************
* Simulation

set seed 1

* Design 1: no difference in Y(0) between takers and non takers
scalar DGP1=1
scalar K1=240				// Number of lotteries
scalar Nk1=40				// Number of applicants per lottery	
scalar compliers1=24		// Number of compliers per lottery	
scalar alwaystakers1=0		// Number of always takers per lottery
scalar Sk1=20				// Number of seats per lottery
scalar delta_AT1=0			// Average difference in y(1) between always takers and compliers
scalar delta_NT1=0			// Average difference in y(0) between never takers and compliers

* Design 2: 0.4 difference in Y(0) between takers and non takers
scalar DGP2=2
scalar K2=240
scalar Nk2=40
scalar compliers2=24
scalar alwaystakers2=0
scalar Sk2=20
scalar delta_AT2=0
scalar delta_NT2=0.4

* Design 3: same as 2, but heterogenous treatment effet: Y(1)-Y(0) | D(1)=1 ~ N(0.2, 1). This is added with condition $DGP==3 in prog simu
scalar DGP3=3
scalar K3=240
scalar Nk3=40
scalar compliers3=24
scalar alwaystakers3=0
scalar Sk3=20
scalar delta_AT3=0
scalar delta_NT3=0.4

forvalue i=1/3{
global DGP=DGP`i'
global K=K`i'
global Nk=Nk`i'
global compliers=compliers`i'
global alwaystakers=alwaystakers`i'
global Sk=Sk`i'
global delta_AT=delta_AT`i'
global delta_NT=delta_NT`i'
global reps=500
simulate 	tslsI=r(tslsI) ///
			coverageI=r(coverageI) ///
			tslsE=r(tslsE) ///
			coverageE=r(coverageE) ///
			tslsDRE=r(tslsDRE) ///
			coverageDRE=r(coverageDRE), saving("${CHEMIN}\log\simulation DGP ${DGP} - ${reps} reps - ${K} strates - ${S_DATE}.dta", replace)reps(${reps}): simu

					
use "${CHEMIN}\log\simulation DGP ${DGP} - ${reps} reps - ${K} strates - ${S_DATE}.dta", clear
save "${CHEMIN}\log\simulation DGP ${DGP} - ${reps} reps.dta", replace
}

* **************************************
* Collecting results

set more off
forvalue dgp=1/3 {
global DGP=`dgp'
global Nk=Nk`dgp'
global compliers=compliers`dgp'

use "${CHEMIN}\log\simulation DGP ${DGP} - ${reps} reps.dta", replace

foreach suf in I E DRE {

qui sum tsls`suf', detail
sca m`suf'=r(mean)
sca med`suf'=r(p50)
sca se`suf'=r(sd)
sca rmse`suf'=((m`suf' - 0.2)^2 + se`suf'^2)^.5
qui sum coverage`suf', detail
sca coverage`suf'=r(mean)
matrix Mtsls`suf'= m`suf', se`suf',coverage`suf'
}
matrix N = MtslsI \ MtslsE \ MtslsDRE
matrix N${DGP}= N
capture log close
log using "${CHEMIN}\log\DGP ${DGP} - ${reps} reps.log", replace
matrix list N
log close

}

matrix list N2

* Compute expected standard errors from theorem 3.3
sca Tk3=compliers3
sca eseDRE= K3 ^(-0.5) * (( 1/(Sk3-1) + 1/(Tk3-Sk3) )/ (Tk3 / Nk3)) ^ (.5)
sca eseI= K3 ^(-0.5) * (( 1/(Sk3) + 1/(Nk3-Sk3) )/ ( ((Tk3-Sk3)/(Nk3-Sk3))^2)) ^ (.5)

* New Table
matrix N=  (N1 \ N2 \ N3), (eseI\.\eseDRE\eseI\.\eseDRE\eseI\.\eseDRE)
matrix list N

outtable using "${CHEMIN}\log\source Simus Revision" , mat(N) replace format(%9.3f)



